﻿' ***********************************************************************
' Assembly         : JiwaSupplierReturnsUI
' Author           : Mike Sheen
' Created          : 10-25-2013
'
' Last Modified By : Mike Sheen
' Last Modified On : 01-06-2014
' ***********************************************************************
' <copyright file="Request.vb" company="Jiwa Financials">
'     Copyright (c) Jiwa Financials. All rights reserved.
' </copyright>
' <summary></summary>
' ***********************************************************************
Option Strict Off
Option Explicit On

Imports System.Drawing
Imports System.Data.SqlClient

''' <summary>
''' The Request namespace.
''' </summary>
Namespace Request
    ''' <summary>
    ''' Class Request.
    ''' </summary>
    Public Class Request
        Implements JiwaApplication.IJiwaDocumentsGrid
        Implements JiwaApplication.IJiwaCustomFieldsGrid
#Region "Events"
        ''' <summary>
        ''' Occurs when [form serial selection form display before].
        ''' </summary>
        Public Event FormSerialSelectionFormDisplayBefore(ByVal sender As Object, ByVal LineDetails As JiwaApplication.Inventory.SOH.LineDetailCollection(Of JiwaSupplierReturns.Request.Line), ByVal OldQuantity As Decimal, ByVal NewQuantity As Decimal)
#End Region

#Region "Fields"
        ''' <summary>
        ''' The _ request
        ''' </summary>
        Private WithEvents _Request As JiwaSupplierReturns.Request.Request
        ''' <summary>
        ''' The _ documents controller
        ''' </summary>
        Private _DocumentsController As JiwaApplication.Documents.DocumentGridController
        ''' <summary>
        ''' The _ notes controller
        ''' </summary>
        Private _NotesController As JiwaApplication.Notes.NoteGridController
        ''' <summary>
        ''' The _ custom fields controller
        ''' </summary>
        Private _CustomFieldsController As JiwaApplication.CustomFields.CustomFieldController
        ''' <summary>
        ''' The displaying
        ''' </summary>
        Public Displaying As Boolean
        ''' <summary>
        ''' The permission view cost price
        ''' </summary>
        Public PermissionViewCostPrice As JiwaApplication.Security.UserGroup.AccessLevels

        Private _LineCustomFieldsController As JiwaApplication.CustomFields.CustomLineFieldController
#End Region

#Region "Properties"
        ''' <summary>
        ''' Gets the description.
        ''' </summary>
        ''' <value>The description.</value>
        Public Overrides ReadOnly Property Description As String
            Get
                Return "Supplier Return Request"
            End Get
        End Property

        ''' <summary>
        ''' Gets or sets the request.
        ''' </summary>
        ''' <value>The request.</value>
        Public Property Request() As JiwaSupplierReturns.Request.Request
            Get
                Return _Request
            End Get
            Set(ByVal value As JiwaSupplierReturns.Request.Request)
                _Request = value
            End Set
        End Property

        ''' <summary>
        ''' Gets the jiwa form.
        ''' </summary>
        ''' <value>The jiwa form.</value>
        Public ReadOnly Property JiwaForm As JiwaApplication.IJiwaForm Implements JiwaApplication.IJiwaDocumentsGrid.JiwaForm, JiwaApplication.IJiwaCustomFieldsGrid.JiwaForm
            Get
                Return Me
            End Get
        End Property

        ''' <summary>
        ''' Gets the documents grid.
        ''' </summary>
        ''' <value>The documents grid.</value>
        Public ReadOnly Property DocumentsGrid As JiwaFinancials.Jiwa.JiwaApplication.Controls.JiwaGrid Implements JiwaApplication.IJiwaDocumentsGrid.DocumentsGrid
            Get
                Return grdDocuments
            End Get
        End Property

        Public ReadOnly Property CustomFieldsGrid As JiwaFinancials.Jiwa.JiwaApplication.Controls.JiwaGrid Implements JiwaApplication.IJiwaCustomFieldsGrid.CustomFieldsGrid
            Get
                Return grdCustomFields
            End Get
        End Property

        ''' <summary>
        ''' Gets the filter string.
        ''' </summary>
        ''' <value>The filter string.</value>
        Public Overrides ReadOnly Property FilterString As String
            Get
                Dim customFilterString As String = MyBase.FilterString
                customFilterString += IIf(customFilterString.Trim.Length > 0, " AND ", " ")

                customFilterString += String.Format(" RA_RequestMain.WarehouseID = '{0}'", Request.Warehouse.IN_LogicalID)

                Return customFilterString
            End Get
        End Property
#End Region

#Region "Setup"
        ''' <summary>
        ''' Setups the before handlers.
        ''' </summary>
        Public Overrides Sub SetupBeforeHandlers()
            MyBase.SetupBeforeHandlers()            
            Me.MdiParent = JiwaApplication.Manager.Instance.MDIParentForm
            SetupObjects()
        End Sub

        ''' <summary>
        ''' Setups this instance.
        ''' </summary>
        Public Overrides Sub Setup()
            MyBase.Setup()
            SetupForm()
            AddHandlers()
        End Sub

        ''' <summary>
        ''' Adds the handlers.
        ''' </summary>
        Public Overrides Sub AddHandlers()
            AddHandler UltraToolbarsManager1.ToolClick, AddressOf UltraToolbarsManager1_ToolClick

            AddHandler RequestNoUltraTextEditor.ValueChanged, AddressOf RequestNoUltraTextEditor_ValueChanged

            AddHandler CreditorLookup.DrillDownRequested, AddressOf CreditorLookup_DrillDownRequested
            AddHandler CreditorLookup.ButtonClicked, AddressOf CreditorLookup_ButtonClicked
            AddHandler ReturnCodeLookup.DrillDownRequested, AddressOf ReturnCodeLookup_DrillDownRequested
            AddHandler ReturnCodeLookup.ButtonClicked, AddressOf ReturnCodeLookup_ButtonClicked

            AddHandler ReferenceUltraTextEditor.ValueChanged, AddressOf ReferenceUltraTextEditor_ValueChanged
            AddHandler StaffMemberLookup.DrillDownRequested, AddressOf StaffMemberLookup_DrillDownRequested
            AddHandler ReturnsWarehouseLookup.DrillDownRequested, AddressOf ReturnsWarehouseLookup_DrillDownRequested
            AddHandler ReturnsWarehouseLookup.ButtonClicked, AddressOf ReturnsWarehouseLookup_ButtonClicked
            AddHandler RequestDateUltraCalendarCombo.ValueChanged, AddressOf RequestDateUltraCalendarCombo_ValueChanged

            AddHandler DeliveryAddressLookup.Click, AddressOf DeliveryAddressLookup_ButtonClicked

            AddHandler txtDeliveryAddressee.ValueChanged, AddressOf txtDeliveryAddressee_ValueChanged
            AddHandler txtDeliveryAddress1.ValueChanged, AddressOf txtDeliveryAddress1_ValueChanged
            AddHandler txtDeliveryAddress2.ValueChanged, AddressOf txtDeliveryAddress2_ValueChanged
            AddHandler txtDeliverySuburb.ValueChanged, AddressOf txtDeliverySuburb_ValueChanged
            AddHandler txtDeliveryState.ValueChanged, AddressOf txtDeliveryState_ValueChanged
            AddHandler txtDeliveryPostCode.ValueChanged, AddressOf txtDeliveryPostCode_ValueChanged
            AddHandler txtDeliveryCountry.ValueChanged, AddressOf txtDeliveryCountry_ValueChanged
            AddHandler txtDeliveryContactName.ValueChanged, AddressOf txtDeliveryContactName_ValueChanged
            AddHandler txtDeliveryNotes.ValueChanged, AddressOf txtDeliveryNotes_ValueChanged
            AddHandler txtDeliveryCourier.ValueChanged, AddressOf txtDeliveryCourier_ValueChanged

            AddHandler ContactNameLookup.Click, AddressOf ContactNameLookup_ButtonClicked

            AddHandler grdLines.ButtonClicked, AddressOf grdLines_ButtonClicked
            AddHandler grdLines.Change, AddressOf grdLines_Change
            AddHandler grdLines.ComboSelChange, AddressOf grdLines_ComboSelChange
            AddHandler grdLines.RowDragMoveCompleted, AddressOf grdLines_RowDragMoveCompleted
            AddHandler grdLines.CellDoubleClick, AddressOf grdLines_CellDoubleClick

            AddHandler Request.ReadEnd, AddressOf Request_Read
            AddHandler Request.CreateEnd, AddressOf Request_Created
            AddHandler Request.PropertyChanged, AddressOf Request_PropertyChanged

            AddHandler Request.Lines.Added, AddressOf Request_Lines_Added
            AddHandler Request.Lines.Changed, AddressOf Request_Lines_Changed
            AddHandler Request.Lines.Removed, AddressOf Request_Lines_Removed

            AddHandler Request.Lines.SerialSelection, AddressOf Request_Lines_SerialSelection

            MyBase.AddHandlers()
        End Sub
#End Region

#Region "Setup"
        ''' <summary>
        ''' Setups the form.
        ''' </summary>
        Public Sub SetupForm()
            SetupWindow()
            DisplaySearchFilters(3018)
            SetupControls()
        End Sub

        ''' <summary>
        ''' Setups the objects.
        ''' </summary>
        Private Sub SetupObjects()
            Request = JiwaApplication.Manager.Instance.BusinessLogicFactory.CreateBusinessLogic(Of JiwaSupplierReturns.Request.Request)(Me)
            MyBase.BusinessLogic = Request
            MyBase.InitialReadMode = JiwaApplication.IJiwaNavigable.ReadModes.Last
        End Sub

        ''' <summary>
        ''' Setups the controls.
        ''' </summary>
        Private Sub SetupControls()
            Dim group As Infragistics.Win.UltraWinToolbars.RibbonGroup
            Dim buttonTool As Infragistics.Win.UltraWinToolbars.ButtonTool

            group = UltraToolbarsManager1.Ribbon.Tabs("Main").Groups("Actions")

            buttonTool = New Infragistics.Win.UltraWinToolbars.ButtonTool("ID_RecordAddFromPurchaseOrder")
            buttonTool.SharedProps.Caption = "Add From PO"
            UltraToolbarsManager1.Tools.Add(buttonTool)
            group.Tools.AddTool("ID_RecordAddFromPurchaseOrder")

            buttonTool = New Infragistics.Win.UltraWinToolbars.ButtonTool("ID_RecordActivate")
            buttonTool.SharedProps.Caption = "Activate"
            UltraToolbarsManager1.Tools.Add(buttonTool)
            group.Tools.AddTool("ID_RecordActivate")

            UltraToolbarsManager1.Tools("ID_RecordUtilitiesCopy").SharedProps.Visible = False

            group = New Infragistics.Win.UltraWinToolbars.RibbonGroup("Other")
            UltraToolbarsManager1.Ribbon.Tabs("Utilities").Groups.Add(group)

            buttonTool = New Infragistics.Win.UltraWinToolbars.ButtonTool("ID_UtilitiesReApplyReturnCode")
            buttonTool.SharedProps.Caption = "Re-Apply Return Code"
            UltraToolbarsManager1.Tools.Add(buttonTool)

            group = UltraToolbarsManager1.Ribbon.Tabs("Utilities").Groups("Other")
            group.Tools.AddTool("ID_UtilitiesReApplyReturnCode")
            UltraToolbarsManager1.Ribbon.Tabs("Utilities").Groups("Other").Tools("ID_UtilitiesReApplyReturnCode").InstanceProps.MinimumSizeOnRibbon = Infragistics.Win.UltraWinToolbars.RibbonToolSize.Normal

            grdShipping.ActiveSheet.ColumnHeader.Rows(1).Height = grdShipping.ActiveSheet.ColumnHeader.Rows(1).Height / 2
            grdShipping.AddColumn("RecID", New FarPoint.Win.Spread.CellType.TextCellType, "RecID", 10, False, False, True, True, 255, False)
            grdShipping.AddColumn("ShipNo", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "Ship No.", 20, False, True, True, True, , True)
            grdShipping.AddColumn("Date", New FarPoint.Win.Spread.CellType.DateTimeCellType, "Date", 20, False, True, True, True)
            grdShipping.AddColumn("RANumber", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "R.A. Number", 20, False, True, True, True)
            grdShipping.AddColumn("Status", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "Status", 20, False, True, True, True)
            grdShipping.SetupComplete()

            grdShipping.ActiveSheet.ColumnHeader.Rows(0).Visible = True
            grdShipping.ActiveSheet.ColumnHeader.Cells(0, 0).Text = "Shipping"
            grdShipping.ActiveSheet.AddColumnHeaderSpanCell(0, 0, 1, grdShipping.ActiveSheet.ColumnCount)

            grdCredits.ActiveSheet.ColumnHeader.Rows(1).Height = grdCredits.ActiveSheet.ColumnHeader.Rows(1).Height / 2
            grdCredits.AddColumn("RecID", New FarPoint.Win.Spread.CellType.TextCellType, "RecID", 10, False, False, True, True, 255, False)
            grdCredits.AddColumn("CreditNo", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "Credit No.", 20, False, True, True, True, , True)
            grdCredits.AddColumn("Date", New FarPoint.Win.Spread.CellType.DateTimeCellType, "Date", 20, False, True, True, True, , True)
            grdCredits.AddColumn("Status", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "Status", 20, False, True, True, True)
            grdCredits.SetupComplete()

            grdCredits.ActiveSheet.ColumnHeader.Rows(0).Visible = True
            grdCredits.ActiveSheet.ColumnHeader.Cells(0, 0).Text = "Credits"
            grdCredits.ActiveSheet.AddColumnHeaderSpanCell(0, 0, 1, grdCredits.ActiveSheet.ColumnCount)

            grdTotals.ActiveSheet.ColumnHeader.Rows(1).Height = grdTotals.ActiveSheet.ColumnHeader.Rows(1).Height / 2
            grdTotals.ActiveSheet.RowHeader.Visible = True
            grdTotals.ActiveSheet.RowHeader.Rows.Count = 2
            grdTotals.ActiveSheet.RowHeader.AutoText = FarPoint.Win.Spread.HeaderAutoText.Blank
            grdTotals.ActiveSheet.RowHeader.Columns.Count = 1
            grdTotals.ActiveSheet.RowHeader.Columns(0).Width = 22 * (grdTotals.Width / 100)
            grdTotals.ActiveSheet.RowHeader.Cells(0, 0).Value = "Local"
            grdTotals.ActiveSheet.RowHeader.Cells(1, 0).Value = "FX"

            grdTotals.AddColumn("ReturnValue", New FarPoint.Win.Spread.CellType.NumberCellType, "Return Value", 38, False, True, True, True, , True)
            grdTotals.AddColumn("SelectedValue", New FarPoint.Win.Spread.CellType.NumberCellType, "Selected Value", 38, False, True, True, True, , True)
            grdTotals.SetupComplete()

            grdTotals.ActiveSheet.RowCount = 2

            SetupMainTab()

            JiwaApplication.Manager.MakeReadOnly(RequestedQtyNumericEditor, True)
            JiwaApplication.Manager.MakeReadOnly(SelectedQtyNumericEditor, True)
            JiwaApplication.Manager.MakeReadOnly(ShipQtyNumericEditor, True)

            CreditorLookup.DrillDown = True
            CreditorLookup.IsClearable = False

            ReturnsWarehouseLookup.DrillDown = True
            ReturnsWarehouseLookup.IsClearable = False

            ReturnCodeLookup.DrillDown = True
            ReturnCodeLookup.IsClearable = False

            StaffMemberLookup.DrillDown = True
            StaffMemberLookup.IsClearable = False
            StaffMemberLookup.LookupEnabled = False

            _DocumentsController = New JiwaApplication.Documents.DocumentGridController(Me, Me, Request)
            _NotesController = New JiwaApplication.Notes.NoteGridController(Me, Request.Notes, grdNotes)
            _CustomFieldsController = New JiwaApplication.CustomFields.CustomFieldController(Me, Me, grdCustomFields, Request, Request.CustomFields, Request)
        End Sub

        ''' <summary>
        ''' Setups the main tab.
        ''' </summary>
        Private Sub SetupMainTab()
            Dim moneyCellType As New JiwaApplication.JiwaManageGrid.JiwaCurrencyCellType
            moneyCellType.DecimalPlaces = Request.SystemSettings.MoneyDecimalPlaces

            Dim currencyCellType As New JiwaApplication.JiwaManageGrid.JiwaCurrencyCellType
            currencyCellType.DecimalPlaces = 6

            Dim percentageCellType As New JiwaApplication.JiwaManageGrid.JiwaCurrencyCellType
            percentageCellType.DecimalPlaces = 2

            Dim returnCodeCell As New FarPoint.Win.Spread.CellType.ComboBoxCellType

            Dim returnCodes As JiwaSupplierReturns.Configuration.ReturnCodeCollection = JiwaApplication.Manager.Instance.BusinessLogicFactory.CreateBusinessLogic(Of JiwaSupplierReturns.Configuration.ReturnCodeCollection)(Nothing)
            returnCodes.Read()
            Dim items As New List(Of String)
            Dim itemsdata As New List(Of String)

            For Each returnCode As JiwaSupplierReturns.Configuration.ReturnCode In returnCodes
                items.Add(returnCode.ReturnCode)
                itemsdata.Add(returnCode.RecID)
            Next

            returnCodeCell.Items = items.ToArray
            returnCodeCell.ItemData = itemsdata.ToArray

            Dim taxRatesCell As New FarPoint.Win.Spread.CellType.ComboBoxCellType
            Dim TaxRatesList As New List(Of String)
            Dim TaxRateIDsList As New List(Of String)

            For Each taxRate As JiwaApplication.JiwaTaxSystemRates.TaxRate In Request.TaxRates
                TaxRateIDsList.Add(taxRate.RecID)
                TaxRatesList.Add(taxRate.Description)
            Next

            taxRatesCell.ItemData = TaxRateIDsList.ToArray
            taxRatesCell.Items = TaxRatesList.ToArray

            grdLines.AddColumn("RecID", New FarPoint.Win.Spread.CellType.TextCellType, "Key", 0, False, False, True, True, 255, False)
            grdLines.AddColumn("OrderNo", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "Order No.", 5, False, True, True, True, , True)
            grdLines.AddColumn("PartNo", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "Part No.", 5, False, True, True, True, , True)
            grdLines.AddColumn("PartNoLookup", New JiwaFinancials.Jiwa.JiwaApplication.JiwaManageGrid.JiwaLookupButtonCellType, "", 10, False, True)
            grdLines.AddColumn("Description", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "Description", 10, False, True, True, True)

            grdLines.AddColumn("ReturnCode", returnCodeCell, "Return Code", 8, False, True, True, True)
            grdLines.AddColumn("ReturnType", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "Return Type", 4, False, True, True, True)

            grdLines.AddColumn("ReduceGoodsValue", New FarPoint.Win.Spread.CellType.CheckBoxCellType, "Reduce Goods Value", 4, False, False, True, True)
            grdLines.AddColumn("ValueInStock", New FarPoint.Win.Spread.CellType.CheckBoxCellType, "Value In Stock", 4, False, False, True, True)
            grdLines.AddColumn("ExchangeGoods", New FarPoint.Win.Spread.CellType.CheckBoxCellType, "Exchange Goods", 4, False, False, True, True)
            grdLines.AddColumn("ShippingGoods", New FarPoint.Win.Spread.CellType.CheckBoxCellType, "Shipping Goods", 4, False, False, True, True)

            grdLines.AddColumn("Quantity", New FarPoint.Win.Spread.CellType.NumberCellType, "Quantity", 4, False, True, True, True, , , , 0)
            grdLines.AddColumn("QuantitySelected", New FarPoint.Win.Spread.CellType.NumberCellType, "Quantity Selected", 4, False, True, True, True, , , , 0)
            grdLines.AddColumn("QuantitySelectedCost", moneyCellType, "Quantity Selected Cost", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("AverageQuantitySelectedCost", moneyCellType, "Average Quantity Selected Cost", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("UnitReturnValue", moneyCellType, "Unit Return Value", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("FXRate", currencyCellType, "FXRate", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("FXUnitReturnValue", New FarPoint.Win.Spread.CellType.NumberCellType, "FX Unit Return Value", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("AverageFXQuantitySelectedCost", New FarPoint.Win.Spread.CellType.NumberCellType, "Average FX Quantity Selected Cost ", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("TotalReturnValue", moneyCellType, "Total Return Value", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("FXTotalReturnValue", New FarPoint.Win.Spread.CellType.NumberCellType, "FX Total Return Value", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("TaxRate", taxRatesCell, "Tax Rate", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("TaxRatePercentage", percentageCellType, "Tax Rate %", 3, False, True, True, True, , , , 0)
            grdLines.AddColumn("TaxAmount", moneyCellType, "Tax Amount", 5, False, True, True, True, , , , 0)
            grdLines.AddColumn("TotalAmountIncGST", moneyCellType, "Total Amount (Inc. GST)", 5, False, True, True, True, , , , 0)

            grdLines.AddColumn("Note", New JiwaApplication.JiwaManageGrid.JiwaTextCellType, "Note", 10, False, True, True, True)

            grdLines.AddColumn("Bin", New JiwaFinancials.Jiwa.JiwaApplication.JiwaManageGrid.JiwaBinButtonCellType, "", 10, False, True, , , , , , , True)

            _LineCustomFieldsController = New JiwaApplication.CustomFields.CustomLineFieldController(Me, Me, grdLines, Request, Request.LineCustomFields, Request.Lines, "RecID")

            grdLines.SetupComplete()

            grdLines.ActiveSheet.RowHeader.ColumnCount = 1
            grdLines.ActiveSheet.RowHeaderVisible = True
            grdLines.AllowRowMove = True
        End Sub

#End Region

#Region "Toolbar / menu handlers"

        ''' <summary>
        ''' Handles the ToolClick event of the UltraToolbarsManager1 control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="Infragistics.Win.UltraWinToolbars.ToolClickEventArgs"/> instance containing the event data.</param>
        Private Sub UltraToolbarsManager1_ToolClick(ByVal sender As Object, ByVal e As Infragistics.Win.UltraWinToolbars.ToolClickEventArgs)
            Select Case e.Tool.Key
                Case "ID_RecordAddFromPurchaseOrder"
                    AddFromPurchaseOrder()
                Case "ID_RecordActivate"
                    ActivateRequest()
                Case "ID_UtilitiesReApplyReturnCode"
                    ReApplyReturnCode()
            End Select
        End Sub

        Public Sub ReApplyReturnCode()
            Dim oldQuantity As Decimal
            For Each existingLine As JiwaSupplierReturns.Request.Line In _Request.Lines
                oldQuantity = existingLine.Quantity
                existingLine.ReturnCode.ReadRecord(_Request.ReturnCode.RecID)
                existingLine.Quantity = oldQuantity
            Next
        End Sub
#End Region

#Region "Business Logic Event Handlers"
        ''' <summary>
        ''' Request_s the read.
        ''' </summary>
        Private Sub Request_Read()
            DisplayRequest()
            DisplayShipping()
            DisplayCredits()
            DisplayLines()
            _LineCustomFieldsController.DisplayCustomSettingValues()
            DisplayTotals()
            _DocumentsController.DisplayDocuments()
            _NotesController.DisplayNotes()
            _CustomFieldsController.DisplayCustomSettings()
            CheckEditStatus()
        End Sub

        ''' <summary>
        ''' Request_s the created.
        ''' </summary>
        Private Sub Request_Created()
            DisplayRequest()
            DisplayShipping()
            DisplayCredits()
            DisplayLines()
            _LineCustomFieldsController.DisplayCustomSettingValues()
            DisplayTotals()
            _DocumentsController.DisplayDocuments()
            _NotesController.DisplayNotes()
            _CustomFieldsController.DisplayCustomSettings()
            CheckEditStatus()

            ' Prompt for a creditor
            Request.Creditor.Search(Me)
        End Sub

        ''' <summary>
        ''' Handles the PropertyChanged event of the Request control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.ComponentModel.PropertyChangedEventArgs"/> instance containing the event data.</param>
        Private Sub Request_PropertyChanged(ByVal sender As Object, ByVal e As System.ComponentModel.PropertyChangedEventArgs)
            DisplayRequest()
            CheckEditStatus()
        End Sub

        ''' <summary>
        ''' Request_s the lines_ added.
        ''' </summary>
        ''' <param name="item">The item.</param>
        Private Sub Request_Lines_Added(item As JiwaSupplierReturns.Request.Line)
            DisplayLine(item, grdLines.ActiveSheet.RowCount - 1)
            grdLines.ActiveSheet.RowCount += 1
            grdLines.LockColumn(False, "PartNoLookup", grdLines.ActiveSheet.RowCount - 1)
            grdLines.LockColumn(False, "PartNo", grdLines.ActiveSheet.RowCount - 1)
            grdLines.LockColumn(True, "Bin", grdLines.ActiveSheet.RowCount - 1)
            DisplayTotals()
            CheckEditStatus()
        End Sub

        ''' <summary>
        ''' Request_s the lines_ changed.
        ''' </summary>
        ''' <param name="item">The item.</param>
        ''' <param name="e">The <see cref="ComponentModel.PropertyChangedEventArgs"/> instance containing the event data.</param>
        Private Sub Request_Lines_Changed(item As JiwaSupplierReturns.Request.Line, e As ComponentModel.PropertyChangedEventArgs)
            DisplayLine(item)
            DisplayTotals()
            CheckEditStatus()
        End Sub

        ''' <summary>
        ''' Request_s the lines_ removed.
        ''' </summary>
        ''' <param name="item">The item.</param>
        Private Sub Request_Lines_Removed(item As JiwaSupplierReturns.Request.Line)
            For row As Integer = 0 To grdLines.ActiveSheet.RowCount - 1
                If grdLines.GridText("RecID", row) = item.RecID Then
                    grdLines.ActiveSheet.RemoveRows(row, 1)
                    Exit For
                End If
            Next
            DisplayTotals()
            CheckEditStatus()
        End Sub

        ''' <summary>
        ''' Request_s the lines_ serial selection.
        ''' </summary>
        ''' <param name="item">The item.</param>
        ''' <param name="QuantityRequested">The quantity requested.</param>
        Private Sub Request_Lines_SerialSelection(ByVal item As JiwaSupplierReturns.Request.Line, ByVal QuantityRequested As Decimal)
            Dim newQuantity As Decimal

            SelectSerialNumbers(item.LineDetails, QuantityRequested, newQuantity)
        End Sub
#End Region

#Region "Control Event Handlers"
        ''' <summary>
        ''' Handles the ValueChanged event of the RequestNoUltraTextEditor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub RequestNoUltraTextEditor_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.RequestNo = RequestNoUltraTextEditor.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the DrillDownRequested event of the CreditorLookup control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub CreditorLookup_DrillDownRequested(ByVal sender As Object, ByVal e As System.EventArgs)
            Dim form As JiwaApplication.IJiwaForm = JiwaApplication.Manager.Instance.FormFactory.CreateForm("JiwaFinancials.Jiwa.JiwaCreditorsUI.frmCreditor")
            form.DrillDownID = Request.Creditor.CreditorID
            form.Start()
        End Sub

        ''' <summary>
        ''' Handles the ButtonClicked event of the CreditorLookup control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub CreditorLookup_ButtonClicked(sender As Object, e As EventArgs)
            Request.Creditor.Search(Me)
        End Sub

        ''' <summary>
        ''' Handles the DrillDownRequested event of the ReturnCodeLookup control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub ReturnCodeLookup_DrillDownRequested(ByVal sender As Object, ByVal e As System.EventArgs)
            Dim form As JiwaApplication.IJiwaForm = JiwaApplication.Manager.Instance.FormFactory.CreateForm("JiwaFinancials.Jiwa.JiwaSupplierReturnsUI.Configuration.ReturnCodes")
            form.Start()
        End Sub

        ''' <summary>
        ''' Handles the ButtonClicked event of the ReturnCodeLookup control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ReturnCodeLookup_ButtonClicked(sender As Object, e As EventArgs)
            Request.ReturnCode.Search(Me)
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the ReferenceUltraTextEditor control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ReferenceUltraTextEditor_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.Reference = ReferenceUltraTextEditor.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the DrillDownRequested event of the StaffMemberLookup control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub StaffMemberLookup_DrillDownRequested(ByVal sender As Object, ByVal e As System.EventArgs)
            Dim form As JiwaApplication.IJiwaForm = JiwaApplication.Manager.Instance.FormFactory.CreateForm("JiwaFinancials.Jiwa.JiwaStaffUI.frmMain")
            form.DrillDownID = Request.CreatedBy.StaffID
            form.Start()
        End Sub

        ''' <summary>
        ''' Handles the DrillDownRequested event of the ReturnsWarehouseLookup control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        Private Sub ReturnsWarehouseLookup_DrillDownRequested(ByVal sender As Object, ByVal e As System.EventArgs)
            Dim form As JiwaApplication.IJiwaForm = JiwaApplication.Manager.Instance.FormFactory.CreateForm("JiwaFinancials.Jiwa.JiwaWarehousesUI.frmMain")
            form.DrillDownID = Request.ReturnsWarehouse.LogicalWarehouseCollection.PhysicalWarehouse.IN_PhysicalID
            form.Start()
        End Sub

        ''' <summary>
        ''' Handles the ButtonClicked event of the ReturnsWarehouseLookup control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ReturnsWarehouseLookup_ButtonClicked(sender As Object, e As EventArgs)
            'Request.ShipWarehouse.LogicalWarehouseCollection.PhysicalWarehouse.PhysicalWarehouseCollection.SearchLogicalWarehouses(Me)
            Dim AOption As JiwaApplication.JiwaSearch.SearchOption
            Dim sSQL As String

            sSQL = "SELECT In_Logical.In_LogicalID, In_Logical.In_PhysicalID, In_Physical.Description," & " In_Logical.Description from In_Logical JOIN In_Physical ON In_Logical.IN_PhysicalID = In_Physical.IN_PhysicalID " '& " JOIN IN_InTransitWH ON In_Logical.In_LogicalID = IN_InTransitWH.In_LogicalID" & " WHERE In_Logical.In_LogicalID <> " & JiwaApplication.Manager.Instance.Database.FormatChar(JiwaApplication.Manager.Instance.Database.CurrentLogicalWarehouseID)
            With JiwaApplication.Manager.Instance.Search
                .Clear()
                .Caption = "Returns Warehouses"
                .CurrentOption = 1

                AOption = New JiwaApplication.JiwaSearch.SearchOption
                With AOption
                    .Title = "In Transit Warehouses"
                    .SQLStr = sSQL
                    .OrderBy = "ORDER BY In_Physical.Description, In_Logical.Description"
                    .AddColumn("logicalid", VariantType.String, "L", 0, 1)
                    .AddColumn("physicalid", VariantType.String, "L", 0, 2)
                    .AddColumn("Physical Warehouse", VariantType.String, "L", 50, 3)
                    .AddColumn("Logical Warehouse", VariantType.String, "L", 50, 4)
                End With
                .AddSearchOption(AOption)
                AOption = Nothing

                If .Show(Me) = Windows.Forms.DialogResult.OK Then
                    If .Results.Count > 0 Then
                        Request.ReturnsWarehouse = JiwaApplication.Manager.Instance.PhysicalWarehouseCollection.FindLogicalWarehouse(.Fields(1).FieldValue)
                    End If
                End If

            End With
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the RequestDateUltraCalendarCombo control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub RequestDateUltraCalendarCombo_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.RequestDate = RequestDateUltraCalendarCombo.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliveryAddressee control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliveryAddressee_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.DeliveryAddress.Addressee = txtDeliveryAddressee.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliveryAddress1 control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliveryAddress1_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.DeliveryAddress.StreetAddress1 = txtDeliveryAddress1.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliveryAddress2 control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliveryAddress2_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.DeliveryAddress.StreetAddress2 = txtDeliveryAddress2.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliverySuburb control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliverySuburb_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.DeliveryAddress.Locality = txtDeliverySuburb.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliveryState control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliveryState_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.DeliveryAddress.Region = txtDeliveryState.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliveryPostCode control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliveryPostCode_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.DeliveryAddress.Postcode = txtDeliveryPostCode.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliveryCountry control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliveryCountry_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.DeliveryAddress.Country = txtDeliveryCountry.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliveryContactName control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliveryContactName_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.ContactName = txtDeliveryContactName.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliveryNotes control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliveryNotes_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.DeliveryAddress.Notes = txtDeliveryNotes.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ValueChanged event of the txtDeliveryCourier control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub txtDeliveryCourier_ValueChanged(sender As Object, e As EventArgs)
            If Displaying = False Then
                Request.DeliveryAddress.CourierDetails = txtDeliveryCourier.Value
            End If
        End Sub

        ''' <summary>
        ''' Handles the ButtonClicked event of the ContactNameLookup control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub ContactNameLookup_ButtonClicked(sender As Object, e As EventArgs)
            Dim AOption As JiwaApplication.JiwaSearch.SearchOption
            Dim WorkStr As String

            ' Delivery contact name
            With JiwaApplication.Manager.Instance.Search
                .Clear()
                .FilterNo = 50037
                .CurrentOption = 1
                .Caption = "Debtor With Contacts"
                .UsePinBoard = False
            End With

            AOption = New JiwaApplication.JiwaSearch.SearchOption
            With AOption
                WorkStr = "SELECT CN_Contact.ContactID, CN_Contact.Title, CN_Contact.FName, CN_Contact.SName"
                WorkStr = WorkStr & " FROM CN_Contact JOIN CN_Main "
                WorkStr = WorkStr & " ON (CN_Main.ProspectID = CN_Contact.ProspectID) "
                WorkStr = WorkStr & " WHERE CN_Main.CreditorID = " & JiwaApplication.Manager.Instance.Database.FormatChar(Request.Creditor.CreditorID)

                .Title = "Creditor With Contacts"
                .SQLStr = WorkStr
                .OrderBy = "ORDER BY CN_Contact.FName"
                .AddColumn("ID", VariantType.String, "", 0, 1)
                .AddColumn("Title", VariantType.String, "", 10, 2)
                .AddColumn("FName", VariantType.String, "", 40, 3)
                .AddColumn("SName", VariantType.String, "", 50, 4)
            End With
            JiwaApplication.Manager.Instance.Search.AddSearchOption(AOption)
            AOption = Nothing

            With JiwaApplication.Manager.Instance.Search
                If .Show(Me) = Windows.Forms.DialogResult.OK Then
                    If .Results.Count <> 0 Then
                        Request.ContactName = Trim(.Fields(2).FieldValue) & IIf(Trim(.Fields(2).FieldValue).Trim.Length > 0, " ", "") & Trim(.Fields(3).FieldValue) & IIf(Trim(.Fields(3).FieldValue).Trim.Length > 0, " ", "") & Trim(.Fields(4).FieldValue)
                    End If
                End If
            End With
        End Sub

        ''' <summary>
        ''' Handles the ButtonClicked event of the DeliveryAddressLookup control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        Private Sub DeliveryAddressLookup_ButtonClicked(sender As Object, e As EventArgs)
            Dim AOption As JiwaApplication.JiwaSearch.SearchOption

            With JiwaApplication.Manager.Instance.Search
                .Clear()
                .FilterNo = 50038
                .CurrentOption = 1
                .Caption = "Creditor Warehouse Addresses For " & Request.Creditor.Name
                .UsePinBoard = False
            End With

            AOption = New JiwaApplication.JiwaSearch.SearchOption
            With AOption
                .Title = "Address (by postcode)"
                .SQLStr = "SELECT WarehouseID, Description, Address1, Address2, Address3, Address4, PostCode, Country " & " FROM CR_Warehouse " & " WHERE CR_Warehouse.CreditorID = " & JiwaApplication.Manager.Instance.Database.FormatChar(Request.Creditor.CreditorID)

                .OrderBy = "ORDER BY PostCode"
                .AddColumn("WarehouseID", VariantType.String, "L", 0, 1)
                .AddColumn("Description", VariantType.String, "L", 15, 2)
                .AddColumn("Address 1", VariantType.String, "L", 15, 3)
                .AddColumn("Address 2", VariantType.String, "L", 15, 4)
                .AddColumn("Address 3", VariantType.String, "L", 15, 5)
                .AddColumn("Address 4", VariantType.String, "L", 15, 6)
                .AddColumn("Post Code", VariantType.String, "L", 5, 7)
                .AddColumn("Country", VariantType.String, "L", 10, 8)
            End With
            JiwaApplication.Manager.Instance.Search.AddSearchOption(AOption)
            AOption = Nothing

            With JiwaApplication.Manager.Instance.Search
                If .Show(Me) = Windows.Forms.DialogResult.OK Then
                    If .Results.Count <> 0 Then
                        Dim address As JiwaApplication.Entities.Address = Request.Creditor.ReadWarehouseAddress(.Fields(1).FieldValue)

                        Request.DeliveryAddress.StreetAddress1 = address.StreetAddress1
                        Request.DeliveryAddress.StreetAddress2 = address.StreetAddress2
                        Request.DeliveryAddress.Locality = address.Locality
                        Request.DeliveryAddress.Region = address.Region
                        Request.DeliveryAddress.Postcode = address.Postcode
                        Request.DeliveryAddress.Country = address.Country
                        Request.DeliveryAddress.Notes = address.Notes
                        Request.DeliveryAddress.CourierDetails = address.CourierDetails
                    End If
                End If
            End With
        End Sub

#Region "Lines Grid"
        ''' <summary>
        ''' Handles the ButtonClicked event of the grdLines control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="FarPoint.Win.Spread.EditorNotifyEventArgs"/> instance containing the event data.</param>
        Private Sub grdLines_ButtonClicked(sender As Object, e As FarPoint.Win.Spread.EditorNotifyEventArgs)
            Dim column As FarPoint.Win.Spread.Column = grdLines.ActiveSheet.Columns(e.Column)
            Dim ColID As String = column.Tag
            Dim RecID As String = grdLines.GridText("RecID", e.Row)

            If RecID.Trim.Length > 0 Then
                Select Case UCase(ColID)
                    Case UCase("Bin")
                        Request.Lines.Remove(Request.Lines(RecID))
                End Select
            Else
                Select Case UCase(ColID)
                    Case UCase("PartNoLookup")
                        Dim filterString As String = "IN_Main.BOMObject < 2 AND IN_Main.Status <> 2 AND IN_Main.PhysicalItem <> 0"
                        'Dim line As New JiwaSupplierReturns.Request.Line
                        'line.Inventory.Search(Request.Warehouse, Me, filterString)
                        'Request.Lines.Add(line)
                        'line.UnitReturnValue = line.Inventory.LCost

                        Dim items As List(Of JiwaApplication.Entities.Inventory.Inventory) = JiwaApplication.Entities.Inventory.Inventory.MultiSelectSearch(Request.Warehouse, Me, filterString)
                        If Not items Is Nothing AndAlso items.Count > 0 Then
                            For Each inventory As JiwaApplication.Entities.Inventory.Inventory In items
                                Dim line As New JiwaSupplierReturns.Request.Line
                                line.Inventory.ReadRecord(inventory.InventoryID)
                                Request.Lines.Add(line)
                                line.UnitReturnValue = line.Inventory.LCost
                            Next                            
                        End If

                End Select
            End If
        End Sub

        ''' <summary>
        ''' Handles the Change event of the grdLines control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="FarPoint.Win.Spread.ChangeEventArgs"/> instance containing the event data.</param>
        Private Sub grdLines_Change(sender As Object, e As FarPoint.Win.Spread.ChangeEventArgs)
            Dim RecID As String = grdLines.GridText("RecID", e.Row)
            Dim column As FarPoint.Win.Spread.Column = grdLines.ActiveSheet.Columns(e.Column)
            Dim ColID As String = column.Tag

            If RecID.Trim.Length = 0 Then
                ' Adding a new line                
                Select Case UCase(ColID)
                    Case UCase("PartNo")
                        Try
                            Dim line As New JiwaSupplierReturns.Request.Line
                            line.Inventory.ReadRecordFromPartNo(grdLines.GridText("PartNo", e.Row))
                            Request.Lines.Add(line)
                            line.UnitReturnValue = line.Inventory.LCost

                        Catch ex As JiwaApplication.Exceptions.RecordNotFoundException
                            'line.Inventory.Search(Request.Warehouse, Me, , grdLines.GridText("PartNo", e.Row))                            
                            Dim filterString As String = "IN_Main.BOMObject < 2 AND IN_Main.Status <> 2 AND IN_Main.PhysicalItem <> 0"
                            Dim items As List(Of JiwaApplication.Entities.Inventory.Inventory) = JiwaApplication.Entities.Inventory.Inventory.MultiSelectSearch(Request.Warehouse, Me, FilterString, grdLines.GridText("PartNo", e.Row))
                            If Not items Is Nothing AndAlso items.Count > 0 Then
                                For Each inventory As JiwaApplication.Entities.Inventory.Inventory In items
                                    Dim line As New JiwaSupplierReturns.Request.Line
                                    line.Inventory.ReadRecord(inventory.InventoryID)
                                    Request.Lines.Add(line)
                                    line.UnitReturnValue = line.Inventory.LCost
                                Next
                            End If
                        End Try

                End Select
            Else
                ' Editing existing stage
                Select Case UCase(ColID)
                    Case UCase("Quantity")
                        Request.Lines(RecID).Quantity = grdLines.GridText("Quantity", e.Row)
                    Case UCase("UnitReturnValue")
                        Request.Lines(RecID).UnitReturnValue = grdLines.GridText("UnitReturnValue", e.Row)
                    Case UCase("TotalReturnValue")
                        Request.Lines(RecID).TotalReturnValue = grdLines.GridText("TotalReturnValue", e.Row)
                    Case UCase("FXUnitReturnValue")
                        Request.Lines(RecID).FXUnitReturnValue = grdLines.GridText("FXUnitReturnValue", e.Row)
                    Case UCase("FXTotalReturnValue")
                        Request.Lines(RecID).FXTotalReturnValue = grdLines.GridText("FXTotalReturnValue", e.Row)
                    Case UCase("FXRate")
                        Request.Lines(RecID).FXRate = grdLines.GridText("FXRate", e.Row)
                End Select
            End If
        End Sub

        ''' <summary>
        ''' Handles the ComboSelChange event of the grdLines control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="FarPoint.Win.Spread.EditorNotifyEventArgs"/> instance containing the event data.</param>
        Private Sub grdLines_ComboSelChange(ByVal sender As Object, ByVal e As FarPoint.Win.Spread.EditorNotifyEventArgs)
            Dim RecID As String = grdLines.GridText("RecID", e.Row)
            Dim column As FarPoint.Win.Spread.Column = grdLines.ActiveSheet.Columns(e.Column)
            Dim ColID As String = column.Tag

            If Not RecID Is Nothing AndAlso RecID.Trim.Length > 0 Then
                Select Case UCase(ColID)
                    Case UCase("ReturnCode")
                        Dim oldQuantity As Decimal = Request.Lines(RecID).Quantity
                        Request.Lines(RecID).ReturnCode.ReadRecord(grdLines.GridText(ColID, e.Row))
                        Request.Lines(RecID).Quantity = oldQuantity
                    Case UCase("TaxRate")
                        Request.Lines(RecID).TaxRate.Read(grdLines.GridText(ColID, e.Row))
                End Select
            End If

        End Sub

        ''' <summary>
        ''' Handles the RowDragMoveCompleted event of the grdLines control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="FarPoint.Win.Spread.DragMoveCompletedEventArgs"/> instance containing the event data.</param>
        Private Sub grdLines_RowDragMoveCompleted(sender As Object, e As FarPoint.Win.Spread.DragMoveCompletedEventArgs)

            For row As Integer = 0 To grdLines.ActiveSheet.RowCount - 1
                Dim recID As String = grdLines.GridText("RecID", row).ToString

                If recID.Trim.Length > 0 Then
                    Request.Lines(recID).ItemNo = row + 1
                End If
            Next
        End Sub

        ''' <summary>
        ''' Handles the CellDoubleClick event of the grdLines control.
        ''' </summary>
        ''' <param name="sender">The source of the event.</param>
        ''' <param name="e">The <see cref="FarPoint.Win.Spread.CellClickEventArgs"/> instance containing the event data.</param>
        Private Sub grdLines_CellDoubleClick(sender As Object, e As FarPoint.Win.Spread.CellClickEventArgs)
            If e.ColumnHeader = False AndAlso e.Button = System.Windows.Forms.MouseButtons.Right Then
                With grdLines
                    Dim recID As String = .GridText("RecID", e.Row)
                    If recID.Trim.Length > 0 Then
                        Dim line As JiwaSupplierReturns.Request.Line = Request.Lines(recID)

                        If Not line Is Nothing Then
                            Select Case grdLines.ActiveSheet.Columns(e.Column).Tag
                                Case "QuantitySelected"
                                    Try
                                        SelectSerialNumbers(line.LineDetails, line.Quantity, line.Quantity, e.Row, , True)
                                    Catch ex As JiwaApplication.Exceptions.ClientCancelledException
                                        ' User cancelled serial selection - reset to original value
                                        If e.Row <> -1 Then
                                            grdLines.GridText("QuantitySelected", e.Row) = line.QuantityTaken
                                        End If
                                    End Try
                            End Select
                        End If
                    End If
                End With
            End If
        End Sub
#End Region
#End Region

#Region "Display"
        ''' <summary>
        ''' Displays the request.
        ''' </summary>
        Public Sub DisplayRequest()
            Dim oldDisplaying As Boolean = Displaying

            Try
                Displaying = True

                UltraStatusBar1.Panels(0).Text = "Warehouse : " & Request.Warehouse.ToString
                UltraStatusBar1.Appearance.BackColor = Request.Warehouse.Colour
                UltraStatusBar1.PanelAppearance.BackColorAlpha = Infragistics.Win.Alpha.Transparent
                UltraStatusBar1.PanelAppearance.BackColor = Request.Warehouse.Colour
                UltraStatusBar1.PanelAppearance.BorderAlpha = Infragistics.Win.Alpha.Transparent

                RequestNoUltraTextEditor.Text = Request.RequestNo
                CreditorLookup.Text = Request.Creditor.AccountNo & " - " & Request.Creditor.Name
                ReturnCodeLookup.Text = Request.ReturnCode.Description
                ReferenceUltraTextEditor.Text = Request.Reference
                StaffMemberLookup.Text = Request.CreatedBy.DisplayName
                RequestDateUltraCalendarCombo.Value = Request.RequestDate
                ReturnsWarehouseLookup.Text = Request.ReturnsWarehouse.ToString

                txtDeliveryAddressee.Text = Request.DeliveryAddress.Addressee
                txtDeliveryAddress1.Text = Request.DeliveryAddress.StreetAddress1
                txtDeliveryAddress2.Text = Request.DeliveryAddress.StreetAddress2
                txtDeliverySuburb.Text = Request.DeliveryAddress.Locality
                txtDeliveryState.Text = Request.DeliveryAddress.Region
                txtDeliveryPostCode.Text = Request.DeliveryAddress.Postcode
                txtDeliveryCountry.Text = Request.DeliveryAddress.Country
                txtDeliveryContactName.Text = Request.ContactName
                txtDeliveryNotes.Text = Request.DeliveryAddress.Notes
                txtDeliveryCourier.Text = Request.DeliveryAddress.CourierDetails

                lblActivated.Visible = (Request.Status = JiwaSupplierReturns.Request.Request.Statuses.Activated)

                UltraToolbarsManager1.Tools("ID_RecordAddFromPurchaseOrder").SharedProps.Enabled = (Request.Status = JiwaSupplierReturns.Request.Request.Statuses.Open)
                UltraToolbarsManager1.Tools("ID_RecordActivate").SharedProps.Enabled = (Request.Status = JiwaSupplierReturns.Request.Request.Statuses.Open)
                UltraToolbarsManager1.Tools("ID_UtilitiesReApplyReturnCode").SharedProps.Enabled = (Request.Status = JiwaSupplierReturns.Request.Request.Statuses.Open)

                JiwaApplication.Manager.MakeReadOnly(RequestNoUltraTextEditor, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(CreditorLookup, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(ReturnCodeLookup, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(ReferenceUltraTextEditor, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(RequestDateUltraCalendarCombo, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(ReturnsWarehouseLookup, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))

                JiwaApplication.Manager.MakeReadOnly(txtDeliveryAddressee, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(txtDeliveryAddress1, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(txtDeliveryAddress2, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(txtDeliverySuburb, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(txtDeliveryState, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(txtDeliveryPostCode, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(txtDeliveryCountry, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(txtDeliveryContactName, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(txtDeliveryNotes, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))
                JiwaApplication.Manager.MakeReadOnly(txtDeliveryCourier, (Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open))

            Finally
                Displaying = oldDisplaying
            End Try

        End Sub

        ''' <summary>
        ''' Displays the lines.
        ''' </summary>
        Public Sub DisplayLines()
            grdLines.ActiveSheet.RowCount = 0

            For Each line As JiwaSupplierReturns.Request.Line In Request.Lines
                grdLines.ActiveSheet.RowCount += 1
                DisplayLine(line, grdLines.ActiveSheet.RowCount - 1)
            Next

            If Request.Status = JiwaSupplierReturns.Request.Request.Statuses.Open Then
                ' Add blank row for a new row template
                grdLines.ActiveSheet.RowCount += 1
                grdLines.LockColumn(False, "PartNoLookup", grdLines.ActiveSheet.RowCount - 1)
                grdLines.LockColumn(False, "PartNo", grdLines.ActiveSheet.RowCount - 1)
                grdLines.LockColumn(True, "Bin", grdLines.ActiveSheet.RowCount - 1)
            End If

        End Sub

        ''' <summary>
        ''' Displays the line.
        ''' </summary>
        ''' <param name="Line">The line.</param>
        ''' <param name="Row">The row.</param>
        Public Sub DisplayLine(ByVal Line As JiwaSupplierReturns.Request.Line, Optional ByVal Row As Integer = -1)
            If Row = -1 Then
                For lineRow As Integer = 0 To grdLines.ActiveSheet.RowCount - 1
                    If grdLines.GridText("RecID", lineRow) = Line.RecID Then
                        Row = lineRow
                        Exit For
                    End If
                Next
            End If

            If Row <> -1 Then
                ' copy col header tag into each cell tag.
                For Each column As FarPoint.Win.Spread.Column In grdLines.ActiveSheet.Columns
                    grdLines.ActiveSheet.Cells(Row, column.Index).Tag = grdLines.ActiveSheet.ColumnHeader.Cells(1, column.Index).Tag
                Next

                grdLines.GridText("RecID", Row) = Line.RecID

                If Not Line.PurchaseOrderLine Is Nothing Then
                    grdLines.GridText("OrderNo", Row) = Line.PurchaseOrderLine.Lines.PurchaseOrder.OrderNo
                    grdLines.SetDrillDown("OrderNo", Row, "JiwaFinancials.Jiwa.JiwaPurchaseOrdersUI.PurchaseOrders", Line.PurchaseOrderLine.Lines.PurchaseOrder.RecID)
                Else
                    grdLines.GridText("OrderNo", Row) = ""
                End If

                grdLines.GridText("PartNo", Row) = Line.Inventory.PartNo
                grdLines.SetDrillDown("PartNo", Row, "JiwaFinancials.Jiwa.JiwaInventoryUI.InventoryMaintenanceForm", Line.Inventory.InventoryID)

                grdLines.GridText("Description", Row) = Line.Inventory.Description
                grdLines.GridText("ReturnCode", Row) = Line.ReturnCode.RecID
                grdLines.GridText("ReturnType", Row) = Line.ReturnCode.ReturnType

                grdLines.GridText("ReduceGoodsValue", Row) = Line.ReturnCode.ReduceGoodsValue
                grdLines.GridText("ValueInStock", Row) = Line.ReturnCode.ValueInStock
                grdLines.GridText("ExchangeGoods", Row) = Line.ReturnCode.ExchangeGoods
                grdLines.GridText("ShippingGoods", Row) = Line.ReturnCode.ShippingGoods

                grdLines.GridDecimalPlaces("Quantity", Row, Line.Inventory.DecimalPlaces)
                grdLines.GridText("Quantity", Row) = Line.Quantity

                grdLines.GridDecimalPlaces("QuantitySelected", Row, Line.Inventory.DecimalPlaces)
                grdLines.GridText("QuantitySelected", Row) = Line.QuantityTaken

                grdLines.GridText("QuantitySelectedCost", Row) = Line.LineDetails.Value
                grdLines.GridText("AverageQuantitySelectedCost", Row) = Line.LineDetails.AverageValue

                grdLines.GridText("UnitReturnValue", Row) = Line.UnitReturnValue
                grdLines.GridText("FXRate", Row) = Line.FXRate

                grdLines.GridDecimalPlaces("FXUnitReturnValue", Row, Request.Creditor.Currency.DecimalPlaces)
                grdLines.GridText("FXUnitReturnValue", Row) = Line.FXUnitReturnValue

                grdLines.GridDecimalPlaces("AverageFXQuantitySelectedCost", Row, Request.Creditor.Currency.DecimalPlaces)
                grdLines.GridText("AverageFXQuantitySelectedCost", Row) = Line.LineDetails.AverageValue * Line.FXRate

                grdLines.GridText("TotalReturnValue", Row) = Line.TotalReturnValue

                grdLines.GridDecimalPlaces("FXTotalReturnValue", Row, Request.Creditor.Currency.DecimalPlaces)
                grdLines.GridText("FXTotalReturnValue", Row) = Line.FXTotalReturnValue

                grdLines.GridText("TaxRate", Row) = Line.TaxRate.RecID
                grdLines.GridText("TaxRatePercentage", Row) = Line.TaxRate.TaxRate
                grdLines.GridText("TaxAmount", Row) = Line.TaxAmount
                grdLines.GridText("TotalAmountIncGST", Row) = Line.LineTotalIncTax
                grdLines.GridText("Note", Row) = Line.Remark

                _LineCustomFieldsController.DisplayCustomSettingValues(Line)

                grdLines.LockColumn(True, "PartNo", Row)
                grdLines.LockColumn(True, "PartNoLookup", Row)

                grdLines.LockColumn(Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open, "ReturnCode", Row)
                grdLines.LockColumn(Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open, "Quantity", Row)
                grdLines.LockColumn(Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open, "UnitReturnValue", Row)
                grdLines.LockColumn(Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open OrElse Request.Creditor.UsesFX = True, "TaxRate", Row)
                grdLines.LockColumn(Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open, "TotalReturnValue", Row)
                grdLines.LockColumn(Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open, "Note", Row)
                grdLines.LockColumn(Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open, "Bin", Row)

                grdLines.LockColumn((Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open) OrElse Request.Creditor.UsesFX = False, "FXUnitReturnValue", Row)
                grdLines.LockColumn((Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open) OrElse Request.Creditor.UsesFX = False, "FXTotalReturnValue", Row)
                grdLines.LockColumn((Request.Status <> JiwaSupplierReturns.Request.Request.Statuses.Open) OrElse Request.Creditor.UsesFX = False, "FXRate", Row)
            End If

        End Sub

        ''' <summary>
        ''' Displays the shipping.
        ''' </summary>
        Public Sub DisplayShipping()
            grdShipping.ActiveSheet.RowCount = 0

            For Each shipping As JiwaSupplierReturns.Entities.Shipping.Shipping In Request.ShippingCollection
                grdShipping.ActiveSheet.RowCount += 1
                DisplayShip(shipping, grdShipping.ActiveSheet.RowCount - 1)
            Next
        End Sub

        ''' <summary>
        ''' Displays the ship.
        ''' </summary>
        ''' <param name="Line">The line.</param>
        ''' <param name="Row">The row.</param>
        Public Sub DisplayShip(ByVal Line As JiwaSupplierReturns.Entities.Shipping.Shipping, Optional ByVal Row As Integer = -1)
            If Row = -1 Then
                For lineRow As Integer = 0 To grdShipping.ActiveSheet.RowCount - 1
                    If grdShipping.GridText("RecID", lineRow) = Line.RecID Then
                        Row = lineRow
                        Exit For
                    End If
                Next
            End If

            If Row <> -1 Then
                ' copy col header tag into each cell tag.
                For Each column As FarPoint.Win.Spread.Column In grdShipping.ActiveSheet.Columns
                    grdShipping.ActiveSheet.Cells(Row, column.Index).Tag = grdShipping.ActiveSheet.ColumnHeader.Cells(1, column.Index).Tag
                Next

                grdShipping.GridText("RecID", Row) = Line.RecID
                grdShipping.GridText("ShipNo", Row) = Line.ShippingNo
                grdShipping.SetDrillDown("ShipNo", Row, "JiwaFinancials.Jiwa.JiwaSupplierReturnsUI.Shipping.Shipping", Line.RecID)
                grdShipping.GridText("Date", Row) = Line.ShippingDate
                grdShipping.GridText("RANumber", Row) = Line.RANumber

                Select Case Line.Status
                    Case JiwaSupplierReturns.Request.Request.Statuses.Open
                        grdShipping.GridText("Status", Row) = "Open"
                    Case JiwaSupplierReturns.Request.Request.Statuses.Activated
                        grdShipping.GridText("Status", Row) = "Activated"
                    Case JiwaSupplierReturns.Request.Request.Statuses.Closed
                        grdShipping.GridText("Status", Row) = "Closed"
                End Select

            End If

        End Sub

        ''' <summary>
        ''' Displays the credits.
        ''' </summary>
        Public Sub DisplayCredits()
            grdCredits.ActiveSheet.RowCount = 0

            For Each credit As JiwaSupplierReturns.Entities.Credit.Credit In Request.Credits
                grdCredits.ActiveSheet.RowCount += 1
                DisplayCredit(credit, grdCredits.ActiveSheet.RowCount - 1)
            Next
        End Sub

        ''' <summary>
        ''' Displays the credit.
        ''' </summary>
        ''' <param name="Credit">The credit.</param>
        ''' <param name="Row">The row.</param>
        Public Sub DisplayCredit(ByVal Credit As JiwaSupplierReturns.Entities.Credit.Credit, Optional ByVal Row As Integer = -1)
            If Row = -1 Then
                For lineRow As Integer = 0 To grdCredits.ActiveSheet.RowCount - 1
                    If grdCredits.GridText("RecID", lineRow) = Credit.RecID Then
                        Row = lineRow
                        Exit For
                    End If
                Next
            End If

            If Row <> -1 Then
                ' copy col header tag into each cell tag.
                For Each column As FarPoint.Win.Spread.Column In grdCredits.ActiveSheet.Columns
                    grdCredits.ActiveSheet.Cells(Row, column.Index).Tag = grdCredits.ActiveSheet.ColumnHeader.Cells(1, column.Index).Tag
                Next

                grdCredits.GridText("RecID", Row) = Credit.RecID
                grdCredits.GridText("CreditNo", Row) = Credit.CreditNo
                grdCredits.SetDrillDown("CreditNo", Row, "JiwaFinancials.Jiwa.JiwaSupplierReturnsUI.Credit.Credit", Credit.RecID)

                grdCredits.GridText("Date", Row) = Credit.CreditDate
                grdCredits.GridText("Status", Row) = Credit.Status.ToString
            End If

        End Sub

        ''' <summary>
        ''' Displays the totals.
        ''' </summary>
        Public Sub DisplayTotals()
            RequestedQtyNumericEditor.Value = Request.Lines.TotalQuantityRequested
            SelectedQtyNumericEditor.Value = Request.Lines.TotalQuantitySelected
            ShipQtyNumericEditor.Value = Request.Lines.TotalShipQuantity

            Dim maxDecDecimals As Integer = 0

            For Each line As JiwaSupplierReturns.Request.Line In Request.Lines
                If line.Inventory.DecimalPlaces > maxDecDecimals Then
                    maxDecDecimals = line.Inventory.DecimalPlaces
                End If
            Next

            RequestedQtyNumericEditor.DecimalPlaces = maxDecDecimals
            SelectedQtyNumericEditor.DecimalPlaces = maxDecDecimals
            ShipQtyNumericEditor.DecimalPlaces = maxDecDecimals

            With grdTotals
                '.GridText("Currency", 0) = "Local"
                '.GridText("Currency", 1) = Request.Creditor.Currency.ShortName

                grdTotals.ActiveSheet.RowHeader.Cells(0, 0).Value = "Local"
                grdTotals.ActiveSheet.RowHeader.Cells(1, 0).Value = If(Request.Creditor.UsesFX, Request.Creditor.Currency.ShortName, "FX")

                .GridText("ReturnValue", 0) = Request.Lines.TotalReturnValue
                .GridText("ReturnValue", 1) = Request.Lines.TotalFXReturnValue
                .GridText("SelectedValue", 0) = Request.Lines.TotalSelectedValue
                .GridText("SelectedValue", 1) = Request.Lines.TotalFXSelectedValue

                .GridDecimalPlaces("ReturnValue", 0, Request.SystemSettings.MoneyDecimalPlaces)
                .GridDecimalPlaces("ReturnValue", 1, Request.Creditor.Currency.DecimalPlaces)

                .GridDecimalPlaces("SelectedValue", 0, Request.SystemSettings.MoneyDecimalPlaces)
                .GridDecimalPlaces("SelectedValue", 1, Request.Creditor.Currency.DecimalPlaces)
            End With

        End Sub
#End Region

#Region "Misc"
        ''' <summary>
        ''' Activates the request.
        ''' </summary>
        Public Sub ActivateRequest()
            Try
                Request.Status = JiwaSupplierReturns.Request.Request.Statuses.Activated
                Request.Save()
            Catch ex As Exception
                ' If there was an error, and it was during an activate - we need to re-read
                ReadRecord(JiwaApplication.IJiwaNavigable.ReadModes.Actual)
                Throw
            End Try
        End Sub

        ''' <summary>
        ''' Adds from purchase order.
        ''' </summary>
        Public Sub AddFromPurchaseOrder()
            ' Search PO's for the current creditor
            Dim PO As New JiwaApplication.Entities.PurchaseOrder.PurchaseOrder
            PO.Search(Request.Creditor.CreditorID, Me)

            Dim results As List(Of JiwaApplication.JiwaSearch.ResultSet) = SearchPurchaseOrderLines(PO, Me)
            If results.Count > 0 Then
                Dim purchaseOrder As JiwaPurchaseOrders.PurchaseOrder
                purchaseOrder = Nothing

                For Each existingPurchaseOrder As JiwaPurchaseOrders.PurchaseOrder In Request.PurchaseOrders
                    If existingPurchaseOrder.RecID = PO.RecID Then
                        purchaseOrder = existingPurchaseOrder
                        Exit For
                    End If
                Next

                If purchaseOrder Is Nothing Then
                    purchaseOrder = JiwaApplication.Manager.Instance.BusinessLogicFactory.CreateBusinessLogic(Of JiwaPurchaseOrders.PurchaseOrder)(Nothing)
                    purchaseOrder.Read(PO.RecID)
                    Request.PurchaseOrders.Add(purchaseOrder)
                End If

                If Not purchaseOrder Is Nothing Then
                    For Each searchResult As JiwaApplication.JiwaSearch.ResultSet In results

                        For Each poLine As JiwaPurchaseOrders.Line In purchaseOrder.Lines
                            If searchResult.Fields(1).FieldValue = poLine.RecID Then

                                Dim line As New JiwaSupplierReturns.Request.Line
                                line.Inventory.ReadRecord(poLine.InventoryID)
                                line.UnitReturnValue = poLine.Cost
                                line.PurchaseOrderLine = poLine
                                line.FXRate = poLine.CurrencyRateUsed
                                line.FXUnitReturnValue = poLine.FXCost
                                line.FXTotalReturnValue = poLine.FXCost * searchResult.Fields(5).FieldValue

                                Request.Lines.Add(line)

                                line.Quantity = searchResult.Fields(5).FieldValue

                                Exit For
                            End If
                        Next
                    Next
                End If

            End If
        End Sub

        ''' <summary>
        ''' Searches the purchase order lines.
        ''' </summary>
        ''' <param name="PurchaseOrder">The purchase order.</param>
        ''' <param name="OwnerForm">The owner form.</param>
        ''' <returns>List{JiwaApplication.JiwaSearch.ResultSet}.</returns>
        Private Function SearchPurchaseOrderLines(ByVal PurchaseOrder As JiwaApplication.Entities.PurchaseOrder.PurchaseOrder, Optional ByVal OwnerForm As System.Windows.Forms.Form = Nothing) As List(Of JiwaApplication.JiwaSearch.ResultSet)
            Dim newSearchOption As JiwaApplication.JiwaSearch.SearchOption
            Dim results As New List(Of JiwaApplication.JiwaSearch.ResultSet)

            With JiwaApplication.Manager.Instance.Search
                .Clear()
                .SetDefaultSearch(JiwaApplication.JiwaSearch.clsSearch.SearchModes.jswUnknown)
                .Caption = "Purchase Order Lines"
                .FilterNo = 50040
                .UsePinBoard = True
                .ShowQuantityInSearchResults = True
            End With

            newSearchOption = New JiwaApplication.JiwaSearch.SearchOption
            With newSearchOption
                .Title = "Order No"
                .SQLStr = "SELECT PO_Lines.OrderLineID, PO_Lines.OrderLineNo, PO_Lines.PartNo, PO_Lines.Description, PO_Lines.Quantity, PO_Lines.SuppliersCost, PO_Lines.DecimalPlaces FROM PO_Lines WHERE PO_Lines.OrderID = " & JiwaApplication.Manager.Instance.Database.FormatChar(PurchaseOrder.RecID)
                .OrderBy = "ORDER BY PO_Lines.OrderLineNo"
                .AddColumn("OrderLineID", vbString, "L", 0, 1)
                .AddColumn("Line No.", vbString, "L", 10, 2)
                .AddColumn("Part No.", vbString, "L", 20, 3)
                .AddColumn("Description", vbString, "L", 40, 4)
                .AddColumn("Quantity", vbDecimal, "L", 15, 5, , , JiwaApplication.JiwaSearch.Column.DecimalPlacesTypes.AnotherColumn, , , 7)
                .AddColumn("Supplier Cost", vbDecimal, "L", 15, 6, , If(PurchaseOrder.Creditor.UsesFX, PurchaseOrder.Creditor.Currency.DecimalPlaces, Request.SystemSettings.MoneyDecimalPlaces), JiwaApplication.JiwaSearch.Column.DecimalPlacesTypes.HardCoded)
            End With
            JiwaApplication.Manager.Instance.Search.AddSearchOption(newSearchOption)
            newSearchOption = Nothing

            If JiwaApplication.Manager.Instance.Search.Show(If(OwnerForm Is Nothing, Me, OwnerForm)) = Windows.Forms.DialogResult.OK Then
                If JiwaApplication.Manager.Instance.Search.Results.Count > 0 Then
                    For MyLoop As Integer = 1 To JiwaApplication.Manager.Instance.Search.Results.Count
                        results.Add(JiwaApplication.Manager.Instance.Search.Results(MyLoop))
                    Next MyLoop
                End If
            End If

            Return results
        End Function

        ''' <summary>
        ''' Prints the record.
        ''' </summary>
        ''' <param name="ReportDefinition">The report definition.</param>
        Public Overrides Sub PrintRecord(ByVal ReportDefinition As JiwaApplication.PrintGroup.FormReports.ReportDefinition)
            Dim PrintUIObject As JiwaApplication.JiwaPrintReportUI.MainForm
            Dim MyLoop As Integer

            PrintUIObject = JiwaApplication.Manager.Instance.CreateReport(ReportDefinition, 1, True, False, True)
            If Not (PrintUIObject Is Nothing) Then
                For MyLoop = 1 To PrintUIObject.PrintReportObject.JiwaReportRangeCollection.Count
                    If UCase(PrintUIObject.PrintReportObject.JiwaPrintFormulaCollection(PrintUIObject.PrintReportObject.JiwaReportRangeCollection(MyLoop).FormulaKey).Name) = UCase("Pass_SP_RA_RequestMain_RecID") Then
                        PrintUIObject.PrintReportObject.JiwaReportRangeCollection(MyLoop).Value = Request.RecID
                    End If
                Next MyLoop

                PrintUIObject.UpdateReport()
                PrintUIObject.chkDrill.Checked = True
                PrintUIObject.PrintReportToScreen()
                PrintUIObject.Show()
            End If
        End Sub

        ''' <summary>
        ''' Searches the specified ownerform.
        ''' </summary>
        ''' <param name="Ownerform">The ownerform.</param>
        Public Overrides Sub Search(ByVal Ownerform As System.Windows.Forms.Form)
            Request.Search(Request.Warehouse, Ownerform)
        End Sub

        ''' <summary>
        ''' Checks the edit status.
        ''' </summary>
        Public Overrides Sub CheckEditStatus() Implements JiwaApplication.IJiwaDocumentsGrid.CheckEditStatus, JiwaApplication.IJiwaCustomFieldsGrid.CheckEditStatus
            MyBase.CheckEditStatus()
        End Sub

        ''' <summary>
        ''' Sets the permissions.
        ''' </summary>
        Overrides Sub SetPermissions()
            MyBase.SetPermissions()

            PermissionViewCostPrice = GetAbstractPermission("inventory - view cost price")

            If EditPermission <> JiwaApplication.Security.UserGroup.AccessLevels.Allow Then
                UltraToolbarsManager1.Tools("ID_RecordAddFromPurchaseOrder").SharedProps.Enabled = False
                UltraToolbarsManager1.Tools("ID_RecordActivate").SharedProps.Enabled = False
            End If
        End Sub

        ''' <summary>
        ''' Selects the serial numbers.
        ''' </summary>
        ''' <param name="LineDetails">The line details.</param>
        ''' <param name="OldQuantity">The old quantity.</param>
        ''' <param name="NewQuantity">The new quantity.</param>
        ''' <param name="Row">The row.</param>
        ''' <param name="NoStockNoShow">if set to <c>true</c> [no stock no show].</param>
        ''' <param name="ManualSelection">if set to <c>true</c> [manual selection].</param>
        ''' <exception cref="JiwaApplication.Exceptions.ClientCancelledException"></exception>
        Public Sub SelectSerialNumbers(ByVal LineDetails As JiwaApplication.Inventory.SOH.LineDetailCollection(Of JiwaSupplierReturns.Request.Line), ByVal OldQuantity As Decimal, ByVal NewQuantity As Decimal, Optional ByVal Row As Integer = -1, Optional ByVal NoStockNoShow As Boolean = False, Optional ByVal ManualSelection As Boolean = True)
            Dim lSerialLineDetail As JiwaSerialNumbersUI.LineDetail
            Dim LineStillUsed As Boolean
            Dim ActualUsed As Decimal
            Dim lineDetail As JiwaApplication.Inventory.SOH.LineDetail(Of JiwaSupplierReturns.Request.Line)
            Dim line As JiwaSupplierReturns.Request.Line = DirectCast(LineDetails.ParentItem, JiwaSupplierReturns.Request.Line)

            ' Do serial number lookup

            Dim SerialSelectionObject As JiwaSerialNumbersUI.frmSerialSelector = JiwaApplication.Manager.Instance.StartDialog(24, Nothing, Me, "SupplierReturnRequest")

            SerialSelectionObject.ShowCost = (PermissionViewCostPrice = JiwaApplication.Security.UserGroup.AccessLevels.Allow)

            If Request.Status = JiwaSupplierReturns.Request.Request.Statuses.Closed Then
                SerialSelectionObject.ReadOnly = True
            End If

            SerialSelectionObject.LogicalWarehouse = Request.Warehouse
            SerialSelectionObject.BinLocationModeLookup = IIf(Request.SystemSettings.BinLocationMode = JiwaSupplierReturns.Request.SystemSettings.BinLocationModes.Lookup, True, False)
            SerialSelectionObject.DefaultBinLocation = LineDetails.DefaultBinLocation
            SerialSelectionObject.PartNo = LineDetails.ParentItem.Inventory.PartNo
            SerialSelectionObject.DefaultQuantity = 1
            SerialSelectionObject.QuantityDecimalPlaces = LineDetails.ParentItem.Inventory.DecimalPlaces

            SerialSelectionObject.MaxQuantity = -1

            ActualUsed = 0
            For Each lineDetail In LineDetails
                If True Then 'lineDetail.InsertFlag Then
                    ActualUsed = ActualUsed + lineDetail.Quantity
                End If
            Next

            SerialSelectionObject.MaxQuantity = ActualUsed + LineDetails.QuantitySOHAvailable

            ' Exclude from the serial selection dialog transactions which are used by another item on the production
            For Each otherLine As JiwaSupplierReturns.Request.Line In DirectCast(LineDetails.ParentItem, JiwaSupplierReturns.Request.Line).Lines
                If Not otherLine.Equals(LineDetails.ParentItem) Then
                    If otherLine.Inventory.InventoryID = LineDetails.ParentItem.Inventory.InventoryID Then
                        For Each lineDetail In otherLine.LineDetails
                            If lineDetail.Quantity > 0 And lineDetail.InsertFlag Then
                                lSerialLineDetail = New JiwaSerialNumbersUI.LineDetail
                                lSerialLineDetail.SOHID = lineDetail.IN_SOH_LinkID
                                lSerialLineDetail.LineDetailID = lineDetail.RecID
                                lSerialLineDetail.Quantity = lineDetail.Quantity
                                lSerialLineDetail.SerialNumber = lineDetail.SerialNo
                                lSerialLineDetail.ExpiryDate = lineDetail.ExpiryDate
                                lSerialLineDetail.BinLocation.Description = lineDetail.BinLocation.Description
                                lSerialLineDetail.DateIn = lineDetail.DateIn
                                lSerialLineDetail.SpecialPrice = lineDetail.SpecialPrice
                                lSerialLineDetail.CostIn = lineDetail.Cost
                                SerialSelectionObject.ExclusionSerialCollection.Add(lSerialLineDetail)
                            End If
                        Next
                    End If
                End If
            Next

            For Each lineDetail In LineDetails
                If lineDetail.Quantity > 0 Then
                    lSerialLineDetail = New JiwaSerialNumbersUI.LineDetail
                    lSerialLineDetail.SOHID = lineDetail.IN_SOH_LinkID
                    lSerialLineDetail.LineDetailID = lineDetail.RecID
                    lSerialLineDetail.Quantity = lineDetail.Quantity
                    lSerialLineDetail.SerialNumber = lineDetail.SerialNo
                    lSerialLineDetail.ExpiryDate = lineDetail.ExpiryDate
                    lSerialLineDetail.BinLocation.Description = lineDetail.BinLocation.Description
                    lSerialLineDetail.DateIn = lineDetail.DateIn
                    lSerialLineDetail.SpecialPrice = lineDetail.SpecialPrice
                    lSerialLineDetail.CostIn = lineDetail.Cost
                    SerialSelectionObject.SelectedSerials.Add(lSerialLineDetail)
                    lSerialLineDetail.InsertFlag = False
                End If

                If (lineDetail.OriginalQuantity - lineDetail.Quantity) > 0 Then
                    ' Include Serials removed on this edit as being available
                    ' Because we have not updated the SOH Table yet
                    lSerialLineDetail = New JiwaSerialNumbersUI.LineDetail
                    lSerialLineDetail.SOHID = lineDetail.IN_SOH_LinkID
                    lSerialLineDetail.LineDetailID = lineDetail.RecID
                    lSerialLineDetail.Quantity = (lineDetail.OriginalQuantity - lineDetail.Quantity)
                    lSerialLineDetail.SerialNumber = lineDetail.SerialNo
                    lSerialLineDetail.ExpiryDate = lineDetail.ExpiryDate
                    lSerialLineDetail.BinLocation.Description = lineDetail.BinLocation.Description
                    lSerialLineDetail.DateIn = lineDetail.DateIn
                    lSerialLineDetail.SpecialPrice = lineDetail.SpecialPrice
                    lSerialLineDetail.CostIn = lineDetail.Cost
                    SerialSelectionObject.AvailableSerials.Add(lSerialLineDetail)
                End If
            Next

            If LineDetails.ParentItem.IsNonStock = False Then
                If SerialSelectionObject.MaxQuantity <> 0 Then
                    RaiseEvent FormSerialSelectionFormDisplayBefore(Me, LineDetails, OldQuantity, NewQuantity)
                    SerialSelectionObject.InventoryID = LineDetails.ParentItem.Inventory.InventoryID
                    SerialSelectionObject.QuantityDecimalPlaces = LineDetails.ParentItem.Inventory.DecimalPlaces
                    SerialSelectionObject.SerialSelectionType = JiwaSerialNumbersUI.frmSerialSelector.SerialSectorTypes.e_SerialSectorInvoice
                    'SerialSelectionObject.NoStockNoShow ?
                    SerialSelectionObject.SerialDisplaySortOrder = Request.SystemSettings.SerialDisplaySortOrder
                End If
            Else
                With JiwaApplication.Manager.Instance.Database
                    Dim Sql As String
                    Dim SQLReader As SqlDataReader = Nothing
                    Dim SQLParam As SqlParameter = Nothing

                    Try
                        For Each lSerialLineDetail In SerialSelectionObject.SelectedSerials
                            lSerialLineDetail.Quantity = 0
                            With JiwaApplication.Manager.Instance.Database

                                Sql = "SELECT QuantityLeft FROM IN_SOH WHERE LinkID = @SOHID"

                                Using SQLCmd As SqlCommand = New SqlCommand(Sql, JiwaApplication.Manager.Instance.Database.SQLConnection, JiwaApplication.Manager.Instance.Database.SQLTransaction)

                                    SQLParam = New SqlParameter("@SOHID", System.Data.SqlDbType.Char)
                                    SQLParam.Value = lSerialLineDetail.SOHID
                                    SQLCmd.Parameters.Add(SQLParam)

                                    SQLReader = SQLCmd.ExecuteReader()

                                    If SQLReader.Read = True Then
                                        lSerialLineDetail.Quantity = .Sanitise(SQLReader, 1)
                                    End If

                                    SQLReader.Close()
                                End Using
                            End With
                        Next

                    Finally
                        If Not SQLReader Is Nothing Then
                            SQLReader.Close()
                        End If
                    End Try

                End With

            End If

            ' TODO : work out what this was for
            'SerialSelectionObject.creditcost = SalesOrder.SalesOrderLines(SalesOrderLineKey).LastCost

            If SerialSelectionObject.MaxQuantity <> 0 Then
                RaiseEvent FormSerialSelectionFormDisplayBefore(Me, LineDetails, OldQuantity, NewQuantity)

                SerialSelectionObject.InventoryID = LineDetails.ParentItem.Inventory.InventoryID
                SerialSelectionObject.QuantityDecimalPlaces = LineDetails.ParentItem.Inventory.DecimalPlaces
                SerialSelectionObject.SerialSelectionType = JiwaSerialNumbersUI.frmSerialSelector.SerialSectorTypes.e_SerialSectorCreditNote
                SerialSelectionObject.SerialDisplaySortOrder = Request.SystemSettings.SerialDisplaySortOrder
                SerialSelectionObject.ReadAvailableSerials()

                If SerialSelectionObject.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
                    ' User has changed serial selection

                    For Each lineDetail In LineDetails
                        LineStillUsed = False
                        For Each lSerialLineDetail In SerialSelectionObject.SelectedSerials
                            If lSerialLineDetail.LineDetailID = lineDetail.RecID Then
                                ' Line Detail Found
                                LineStillUsed = True
                                Exit For
                            End If
                        Next lSerialLineDetail

                        If LineStillUsed = False Then
                            ' Remove the line
                            If lineDetail.InsertFlag Then
                                LineDetails.Remove(lineDetail)
                            Else
                                lineDetail.Quantity = 0
                            End If
                        End If
                    Next

                    'For Each lineDetail In LineDetails
                    '    If lineDetail.InsertFlag Then
                    '        LineDetails.Remove(lineDetail)
                    '    End If
                    'Next

                    For Each lSerialLineDetail In SerialSelectionObject.SelectedSerials
                        If lSerialLineDetail.InsertFlag Then
                            ' New line selected
                            lineDetail = New JiwaApplication.Inventory.SOH.LineDetail(Of JiwaSupplierReturns.Request.Line)
                            lineDetail.BinLocation = lSerialLineDetail.BinLocation
                            lineDetail.Cost = lSerialLineDetail.CostIn
                            lineDetail.ExpiryDate = lSerialLineDetail.ExpiryDate
                            lineDetail.OriginalQuantity = 0
                            lineDetail.Quantity = lSerialLineDetail.Quantity
                            lineDetail.SerialNo = lSerialLineDetail.SerialNumber
                            lineDetail.IN_SOH_LinkID = lSerialLineDetail.SOHID
                            lineDetail.SpecialPrice = lSerialLineDetail.SpecialPrice
                            lineDetail.DateIn = lSerialLineDetail.DateIn
                            lineDetail.IN_LogicalID = Request.Warehouse.IN_LogicalID
                            'lineDetail.SOHInsertFlag = True
                            LineDetails.Add(lineDetail)
                        Else
                            ' Existing line may have been modified - adjust quantity
                            For Each lineDetail In LineDetails
                                If lineDetail.RecID = lSerialLineDetail.LineDetailID Then
                                    lineDetail.Quantity = lSerialLineDetail.Quantity
                                    lineDetail.BinLocation = lSerialLineDetail.BinLocation
                                    lineDetail.SerialNo = lSerialLineDetail.SerialNumber
                                    lineDetail.ExpiryDate = lSerialLineDetail.ExpiryDate
                                    lineDetail.DateIn = lSerialLineDetail.DateIn
                                    lineDetail.SpecialPrice = lSerialLineDetail.SpecialPrice
                                    Exit For
                                End If
                            Next
                        End If
                    Next lSerialLineDetail
                Else
                    Throw New JiwaApplication.Exceptions.ClientCancelledException
                End If
            End If

        End Sub
#End Region

    End Class
End Namespace